


def citation(text, author):
    """
    """

    content = f'''
<figure>
    <blockquote class="blockquote">
        <p>{text}</p>
    </blockquote>
    <figcaption class="blockquote-footer">
        <cite title="{author}">
                {author}
        </cite>
    </figcaption>
</figure>

    '''
    return content



from .link import link

from .image import image
from .video import video

from .js_json_editor import json_editor
from .div import div
from .div_title import div_title

from .title import title
from .table import table
from .card import card
from .page import page
from .button import button
from .pagination import pagination
from .cardgrid import cardgrid
from .carousel import carousel
from .accordion import accordion
from .js_delayed_load import js_delayed_load
from .breadcrumb import breadcrumb
from .panes import panes
from .rating import rating
from .summary import summary
from .section import section
from .paragraph import paragraph
from .code import code
from .anon import anon
from .map import map

from .citation import citation
from .modal import get_modal

from .media import get_video
from .iframe import iframe
from .footer import footer

from .form_input import form_input_text
from .form_input import form_input_url
from .form_input import form_input_tel
from .form_input import form_input_password
from .form_input import form_input_email
from .form_input import form_input_date
from .form_input import form_input
from .form_input import form_input_two

from .list import get_list

from .form import form
from .website import website
from .schema_markup import schema_markup






def page(title, head_content, body_content, foot_content):
    """Wraps html content into web page
    """
    content = f'''
        <!DOCTYPE html>
            <title>
                {title}
            </title>
            <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.2/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-T3c6CoIi6uLrA9TneNEoa7RxnatzjcDSCmG1MXxSR1GAsXEV/Dwwykc2MPK8M2HN" crossorigin="anonymous">
            <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bootstrap-icons@1.11.2/font/bootstrap-icons.min.css">

            <meta charset="utf-8">
            <head>
                {head_content}
                
            </head>
            <body>
                <div class="container-fluid">
                    {body_content}
                </div>
            <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.2/dist/js/bootstrap.bundle.min.js" integrity="sha384-C6RzsynM9kWDrMNeT87bh95OGNyZPhcTNXj1NW7RuBCsyN/o0jlpcV8Qyq46cDfL" crossorigin="anonymous"></script>
            </body>
            <footer>
                {foot_content}
            </footer>
        </html>
        '''

    return content





from kraken_html.library import sections
from kraken_html.library import parts
from kraken_html.library import config

def blank(page_content, language='en'):
    """Blank page, mostly for testing purpose
    """

    if isinstance(page_content, list):
        page_content = ' '.join(page_content)

    
    content = f'''

        <!DOCTYPE html>
        <html  lang="{language}">
        
            <head>
                <meta charset="utf-8" >
                <title>Blank page</title>
                
                <!-- bootstrap css reqs -->
                {config.bootstrap_header()}
        
            </head> 
        
            <body>
        
                <!-- actual content of the web page -->
                {page_content}
        
                <!-- js scripts for bootstrap -->
                {config.bootstrap_body()}
        
            </body>
        
        </html>

    '''
    return content





from flask import Flask
from flask import request
from flask import Response
from flask import redirect
from flask import url_for
from flask import jsonify
from kraken_html.helpers import json

from kraken_html import kraken_html as m
from kraken_html.Class_kraken_html import Html
from kraken_html.Class_kraken_htmls import Htmls

from kraken_schema_org import kraken_schema_org as k


UPLOAD_FOLDER = '/path/to/the/uploads'

# Initialize flask app
app = Flask(__name__,
            static_url_path='',
            static_folder='static',
            template_folder='templates')
app.secret_key = b'_5#mn"F4Q8znxec]/'
app.config['SEND_FILE_MAX_AGE_DEFAULT'] = 0
app.config['UPLOAD_FOLDER'] = UPLOAD_FOLDER


@app.route('/', methods=['GET', 'POST'])
def main_get():



    if 1==1:
        
        action = {
            '@type': 'action',
            'name': 'Test action',
            'url': '/'
        }

        content = m.sections.search(action)

        page_content = m.pages.blank(content)
    
        return Response(page_content)

    
    if 1==0:


        content = ''
        
        record = {
            '@type': "person",
            "@id": "abc123",
            "givenName": "Bob",
            "familyName": "Smith",
            "url": "https://www.test.com",
            'telephone': "15146073437",
            "email": "bob@test.com",
            'title': 'kick_ass in chief', 
            "worksFor": {
                '@type': "organization",
                '@id': "test_org",
                "name": "Test Org",
                "url": "https://www.test_org.com/"
            }
        }


        address= k.get_data('postalAddress')
        content += m.widgets.map(address)

        record = k.get_data('Person')
        record = k.normalize_record(record)
        record['name'] = k.get_record_name(record)

        content += m.widgets.person(record)

        
        url = 'https://www.test.com'
        content += m.form.form(url, 'person', 'EN-US', record)
        
        page_content = m.pages.blank(content)
        
        return Response(page_content)

    
    key = 'name'
    value = request.args.get(key)

    if value:
        r = Html()
        records = r.autocomplete(key, '%' + str(value) + '%')
        return jsonify(records)

    header = {'headline': 'Test headline', 'text': 'some text'}

    content = m.sections.pricing('r')
    content += m.sections.hero(header)
    page_content = m.pages.base(content)
    
    
    #content = "Api for kraken_html"
    return Response(page_content)


@app.route('/<key>/<value>', methods=['GET', 'POST'])
def search_path_get(key, value):

    r = Html()
    records = r.search(key, '%' + value + '%')
    return jsonify(records)


@app.route('/autocomplete', methods=['GET', 'POST'])
def autocomplete_params_get():

    key = 'name'
    value = request.args.get(key)

    r = Html()
    records = r.autocomplete(key, '%' + str(value) + '%')
    return jsonify(records)



@app.route('/autocomplete/<key>/<value>', methods=['GET', 'POST'])
def autocomplete_path_get(key, value):

    r = Html()
    records = r.autocomplete(key, '%' + value + '%')
    return jsonify(records)

@app.route('/test', methods=['GET', 'POST'])
def test_get(key, value):


    content = m.get_catalog()
    page_content = m.pages.blank(content)
    
    return Response(page_content)


def run_api():
    app.run(host='0.0.0.0', debug=False)


    


from kraken_html.library import sections
from kraken_html.library import parts
from kraken_html.library import pages
from kraken_html.library import html_record
from kraken_html.library import form
from kraken_html.library import widgets

#from kraken_html.catalog.catalog import catalog



def get_catalog():
    return catalog()
    




def html_nav(brand, item_links):
    """

    Takes input as ist of dict with text, url
    """

    # Generate menu items
    menu_items_content = get_menu_items(item_links)


    # Geneate full menu
    content = full_menu(brand, menu_items_content)

    return content


def get_menu_items(item_links):

    menu_items_content = ''
    
    for item_link in item_links:
        menu_items_content += get_menu_item_content(item_link['text'], item_link['url'])

    return menu_items_content

def get_menu_item_content(item, link):

    content = '''
        <li class="nav-item">
            <a class="nav-link active" aria-current="page" href="{link}">
                {item}
            </a>
        </li>
        '''.format(
            item = item, 
            link = link
            )
    return content


def full_menu(brand, menu_items_content):
    # Geneate full menu
    content = '''
        <nav class="navbar navbar-expand-md navbar-dark bg-dark">

            <div class="container-fluid">

                <a class="navbar-brand" href="/">
                    {brand}
                </a>

                <button class="navbar-toggler" type="button" data-bs-toggle="collapse" data-bs-target="#navbarNav" aria-controls="navbarNav" aria-expanded="false" aria-label="Toggle navigation">
                    <span class="navbar-toggler-icon"></span>
                </button>

                <div class="collapse navbar-collapse" id="navbarNav">
                    <ul class="navbar-nav">
                
                        {menu_items_content}

                    </ul>
                </div>
            </div>

            <!-- Navbar content -->
        </nav>
        '''.format(
            brand = brand, 
            menu_items_content = menu_items_content
            )

    return content




def form(content, action, action_text='Submit'):
    
    
    content = f'''
        <form action="{action}" method="post">
         {content}
         <input type="submit" value="{action_text}">
        </form>
    '''

    return content








def cardgrid(cards):

    card_html = ''
    for card in cards:
        card_html += '<div class="col mb-3"> {card} </div>'.format(card=card)


    grid_card = '<div class="row">{card_html}</div>'.format(card_html = card_html)

    return grid_card


#row-cols-1 row-cols-md-4 row-cols-auto



def link(url, text, new_tab = None):
    """Converts a url and text into a html link
    """

    if not url:
        return ''
    
    if new_tab:
        link = f'<a href="{url}" target="_blank">{text}</a>'
    else:
        
        link = f'<a href="{url}">{text}</a>'
    return link







def button(url, action_id=None, action_value=None, text=None):
    
    new_url = url

    if action_id and action_value:
        new_url = str(url) + '?' + str(action_id) + '=' + str(action_value)
    
    link = f'<form action="{new_url}" method="post"><button class="btn btn-primary" name="{action_id}" value="{action_value}">{text}</button></form>'


    
    return link



from kraken_schema_org import kraken_schema_org as k
from kraken_html.library import html


def form(url, record_type, language='EN-US', base_record=None, summary=True):
    """
    record_type: te type of the record for the form
    base_record: the initial values (default values) of the form
    summary: if True, returns only summary keys
    """

    base_record = base_record if isinstance(base_record, dict) else {}
    

    # Get fields for record_type

    if summary:
        keys = k.get_summary_keys(record_type)
    else:
        keys = k.get_keys(record_type)

    
    if not keys:
        return ''

    
    #Iterate through keys to generate form fields
    elements = []
    for key in keys:

        
        # Get html type
        html_type = k.key_get_html_types(key)
        html_type = html_type[0] if isinstance(html_type, list) and len(html_type) > 0 else 'text'
        label = key
        value = base_record.get(key, None)

        element = html.form_input(key, label, value, '', '', html_type)
        elements.append(element)

    # Generate form
    form_content = html.form(''.join(elements), url, 'Submit')

    return form_content




def title(text, level = 'H1'):
    content = '''
        <{level} class="pb-xl-1 pt-xl-5">{text}</{level}>
        '''.format(
            text = text,
            level = level
        )
    
    return content





def pricing(offers, theme='dark'):
    """
    """


    offer = {
        '@type': 'offer',
        'headline': 'offer1',
        'text': '',
        'features': ['awesome', 'spectacular', 'extraordinary'],
        'price': '480',
        'cta': 'Get it now'

    }
    offers = [offer, offer, offer, offer]


    content = f'''
    
    <div class="container py-3 bg-{theme} text-bg-{theme}" data-bs-theme="{theme}" data-aos="fade-down" data-aos-delay="200">
    <div class="row row-cols-1 row-cols-md-{str(len(offers))} mb-3 text-center">
      {_get_pricing(offers)}
    </div>
    </div>
    
    '''
   
    return content


def _get_pricing(offers):
    """
    """
    content = ''
    for offer in offers:
        offer_content = f'''
        <div class="col">
            <div class="card mb-4 rounded-3 shadow-sm">
              <div class="card-header py-3">
                <h4 class="my-0 fw-normal">{offer.get('headline', None)}</h4>
              </div>
              <div class="card-body">
                <h1 class="card-title pricing-card-title">${offer.get('price', None)}<small class="text-body-secondary fw-light">/mo</small></h1>
                <ul class="list-unstyled mt-3 mb-4">
                  {_get_features(offer.get('features', None))}
                </ul>
                <button type="button" class="w-100 btn btn-lg btn-primary">{offer.get('cta', None)}</button>
              </div>
            </div>
          </div>
        
        '''
        content += offer_content
    
    return content


def _get_features(features):
    '''
    
    '''
    content = ''
    for i in features:
        content += f'<li>{i}</li>'

    return content





def carousel(content_items):
    """
    """

    carousel_content = ''
    for i in content_items:
        carousel_content += carousel_item(i)
    
    content = '''
    <div id="carouselExampleControls" class="carousel slide" data-bs-ride="carousel">
      <div class="carousel-inner">
        {carousel_content}
      <button class="carousel-control-prev" type="button" data-bs-target="#carouselExampleControls" data-bs-slide="prev">
        <span class="carousel-control-prev-icon" aria-hidden="true"></span>
        <span class="visually-hidden">Previous</span>
      </button>
      <button class="carousel-control-next" type="button" data-bs-target="#carouselExampleControls" data-bs-slide="next">
        <span class="carousel-control-next-icon" aria-hidden="true"></span>
        <span class="visually-hidden">Next</span>
      </button>
    </div>
    '''.format(carousel_content=carousel_content)

    return content

def carousel_item(item_content):
    """
    """

    content = '''
    <div class="carousel-item active">
          {item_content}
    </div>
    '''.format(item_content=item_content)

    return content




def video(url, text = None, thumbnail_url = None):

    
    if isinstance(url, list):
            url = url[0]
        
    if not url or len(url) < 5:
        return ''

   

    html_thumb = '''
        <video class="video-fluid card-img-top z-depth-1" poster="{thumbnail_url} playsinline controls>
        <source src="{url}#t=2">
        </video>
        
    '''.format(url=url, text=text, thumbnail_url = str(thumbnail_url))
   




    return html_thumb


def div_title(html, alignment = 'None', minimum_height=0):

    minimum_height = str(minimum_height)
    
    if alignment == 'right':
        content = f'''
        <div class="container-fluid d-flex pt-2 justify-content-end min-vh-{minimum_height}">
            {html}
        </div>
        '''

    else:
        content = f'''
            <div class="container-fluid pt-2 min-vh-{minimum_height}">
                {html}
            </div>
            '''

    return content








def form_input(name, label, default_value=None, autocomplete_field='', help_message='', input_type='text'):
    
    label_content = f'''
    <label for="{name}" class="form-label" >{label}</label>
    '''

    input_content = f'''<input type="{input_type}" autocomplete="{autocomplete_field}" class="form-control" id="{name}" name="{name}" aria-describedby="{name}Help" value="{str(default_value) if default_value else ''}" placeholder="xxx">'''


    
    content = f'''
      <div class="form-floating mb-3">
        {input_content}
        {label_content if label else ''}
        
        <div id="{name}Help" class="form-text">{help_message}</div>
      </div>

    '''
    

    return content




def form_input_text(name, label, default_value=None, autocomplete_field='', help_message=''):
    input_type = 'text'
    return form_input(name, label, default_value, autocomplete_field, help_message, input_type)

def form_input_date(name, label, default_value=None,autocomplete_field='',  help_message=''):
    input_type = 'date'
    return form_input(name, label, default_value,autocomplete_field,  help_message, input_type)

def form_input_email(name, label, default_value=None, autocomplete_field='',  help_message=''):
    input_type = 'email'
    return form_input(name, label, default_value,autocomplete_field,  help_message, input_type)

def form_input_password(name, label, default_value=None, autocomplete_field='',  help_message=''):
    input_type = 'password'
    return form_input(name, label, default_value,autocomplete_field, help_message, input_type)

def form_input_tel(name, label, default_value=None, autocomplete_field='',  help_message=''):
    input_type = 'tel'
    return form_input(name, label, default_value,autocomplete_field, help_message, input_type)

def form_input_url(name, label, default_value=None, autocomplete_field='',  help_message=''):
    input_type = 'url'
    return form_input(name, label, default_value,autocomplete_field,  help_message, input_type)

def form_input_search(name, label, default_value=None, autocomplete_field='',  help_message=''):
    input_type = 'search'
    return form_input(name, label, default_value, autocomplete_field, help_message, input_type)


def form_input_two(name, label, default_value='', autocomplete_field='', name2='', label2='', default_value2='', autocomplete_field2='', help_message='', input_type='text'):
    
    
    content = f'''
      <div class="row mb-3">
        <div class="col">
            <label for="{name}" class="form-label">{label}</label>
            <input type="{input_type}" autocomplete="{autocomplete_field}" class="form-control" id="{name}" name="{name}" aria-describedby="{name}Help" value="{str(default_value) if default_value else ''}">
        
        </div>
        <div class="col">
            <label for="{name2}" class="form-label">{label2}</label>
            <input type="{input_type}" autocomplete="{autocomplete_field2}" class="form-control" id="{name2}" name="{name2}" aria-describedby="{name2}Help" value="{str(default_value2) if default_value2 else ''}">
        </div>
        <div id="{name}Help" class="form-text">{help_message}</div>
        
      </div>

    '''

    return content



"""
<input type="button">
<input type="checkbox">
<input type="color">
<input type="date">
<input type="datetime-local">
<input type="email">
<input type="file">
<input type="hidden">
<input type="image">
<input type="month">
<input type="number">
<input type="password">
<input type="radio">
<input type="range">
<input type="reset">
<input type="search">
<input type="submit">
<input type="tel">
<input type="text">
<input type="time">
<input type="url">
<input type="week">
"""


'''
"name"
"honorific-prefix"
"given-name"
"additional-name"
"family-name"
"honorific-suffix"
"nickname"
"username"
"new-password"
"current-password"
"one-time-code"
"organization-title"
"organization"
"street-address"
"address-line1"
"address-line2"
"address-line3"
"address-level4"
"address-level3"
"address-level2"
"address-level1"
"country"
"country-name"
"postal-code"
"cc-name"
"cc-given-name"
"cc-additional-name"
"cc-family-name"
"cc-number"
"cc-exp"
"cc-exp-month"
"cc-exp-year"
"cc-csc"
"cc-type"
"transaction-currency"
"transaction-amount"
"language"
"bday"
"bday-day"
"bday-month"
"bday-year"
"sex"
"url"
"photo"


'''



def panes(items):

    element_content = ''
    for i in items:
        element_content += f'<div class="col-lg-4">{i}</div>'
    
    content = f'''
    
    <div class="row">{element_content}</div>
    
    
    '''

    return content


from kraken_html.library.config.bootstrap_header import bootstrap_header

from kraken_html.library.config.bootstrap_body import bootstrap_body
from kraken_html.library.config.schema_record import schema_record






def pagination(url, offset = 0, limit = 20, url_args = {}):

    
    try:
        offset = int(offset)
    except:
        a=1

    try:
        limit = int(limit)
    except:
        a=1

    buttons = []

    current_page = int((offset / limit ) + 1)
    
    # Back button (<<)
    url_args['limit'] = limit    
    
    url_args['offset'] = (int(offset) - int(limit)) if current_page > 1 else 0
    url_button = get_url_with_args(url, url_args)
    name = '&laquo;'
    active = False  
    buttons.append(get_nav_button(url_button, name, active))

    # 1 button
    url_args['limit'] = limit    
    url_args['offset'] = 0
    url_button = get_url_with_args(url, url_args)
    name = '1'
    active = False
    if offset == 0:
        active = True    
    buttons.append(get_nav_button(url_button, name, active))

    # ... button
    if current_page > 3:
        url_args['limit'] = limit    
        url_args['offset'] = 0
        url_button = '#'
        name = '...'
        active = False
        if offset == 0:
            active = True    
        buttons.append(get_nav_button(url_button, name, active))

    # -1 button
    if current_page > 2:
        url_args['limit'] = limit    
        url_args['offset'] =  offset - limit
        url_button = get_url_with_args(url, url_args)
        name = current_page - 1
        active = False
        if offset == 0:
            active = True    
        buttons.append(get_nav_button(url_button, name, active))

    # current button
    if current_page > 1:
        url_args['limit'] = limit    
        url_args['offset'] = offset
        url_button = get_url_with_args(url, url_args)
        name = current_page
        active = True 
        buttons.append(get_nav_button(url_button, name, active))

    # +1 button
    url_args['limit'] = limit    
    url_args['offset'] = offset + limit
    url_button = get_url_with_args(url, url_args)
    name = current_page + 1
    active = False
    buttons.append(get_nav_button(url_button, name, active))

    # +2 button
    if current_page ==1:
        url_args['limit'] = limit    
        url_args['offset'] = offset + (2 * limit)
        url_button = get_url_with_args(url, url_args)
        name = current_page + 2
        active = False
        buttons.append(get_nav_button(url_button, name, active))
        
    # Next button (>>)
    url_args['limit'] = limit    
    url_args['offset'] = int(offset) + int(limit)
    url_button = get_url_with_args(url, url_args)
    name = '&raquo;'
    active = False    
    buttons.append(get_nav_button(url_button, name, active))
    

    buttons_html = ' '.join(buttons)

    content = '''
                <div class="d-flex justify-content-end">
                    <nav aria-label="Page navigation example">
                    <ul class="pagination">
                        {buttons}
                    </ul>
                    </nav> 
                </div>
            '''.format(
                buttons=buttons_html
                )

    return content


def get_url_with_args(base_url, url_args):

    args = []
    for key in url_args:
        arg = key + '=' + str(url_args[key])
        args.append(arg)

    full_url = base_url + '?' + '&'.join(args)

    return full_url


def get_nav_button(url, name, active=False):
        """
        """

        active_class = ''
        if active:
            active_class = 'active'
        
        
        nav_button = '''
        <li class="page-item"><a class="page-link {active_class}" href="{url}">{name}</a></li>
        '''.format(url=url, name=name, active_class = active_class)
    
        return nav_button



from kraken_html.library import html


def person(record, mode='light', language='EN-US'):
    """
    """



    # 
    image_url = record.get('image', None)
    givenName = record.get('givenName', None)
    familyName = record.get('familyName', None)
    title = record.get('title', None)
    email = record.get('email', None)
    phone = record.get('telephone', None)

    worksFor = record.get('worksFor', {})
    worksFor_name = worksFor.get('name', None)
    worksFor_url = worksFor.get('url', None)
    
    # Get profile picture
    profile_picture = html.image(image_url, None) if image_url else html.anon()
    
    # Apply styling
    worksFor_url = html.link(worksFor_url, worksFor_url, True)
    title = f'<strong>{title}</strong>'

    # Generate list
    contact_content = ''
    contact_content += _get_list_element(title)
    contact_content += _get_list_element(worksFor_name)
    contact_content += _get_list_element(phone)
    contact_content += _get_list_element(email)
    contact_content += _get_list_element(worksFor_url)

    
    # Generate content
    content = f'''
<div class="card mb-3" style="max-width: 400px;">
  <div class="row g-0 align-items-center">
    <div class="col-md-4">
    <div class="container text-center">
      {profile_picture}
      </div>
    </div>
    <div class="col-md-8">
      <div class="card-body">
        <h5 class="card-title">{str(givenName)} {str(familyName)}</h5>
        <ul class="list-unstyled">
            {contact_content}
        </ul>
        </small></p>
      </div>
    </div>
  </div>
</div>
    '''
    return content



def _get_list_element(content):
    """
    """

    if not content or content == '':
        return ''

    content = str(content)
    
    content = f'''
    <li>{content}</li>
    '''
    return content






import uuid

def js_delayed_load(url):
    """Returns component for delayed load
    """
    
    div_id = str(uuid.uuid4()).replace('-', '')

    content = '''
        <div id="{div_id}"> </div>
        <script>
        
        async function load_{div_id}() {{
            try {{
                const response = await fetch("{url}")
                const text = await response.text()
                document.getElementById("{div_id}").innerHTML = text
            }}
            catch(err) {{
                document.getElementById("{div_id}").innerHTML = "Error " + err
            }}
            }}
             
            
        load_{div_id}()
        
       
       
         </script>
         '''.format(div_id=div_id, url=url)
    return content




def bootstrap_body():

    content = '''

<!-- Bootstrap js -->
<script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.2/dist/js/bootstrap.bundle.min.js" integrity="sha384-C6RzsynM9kWDrMNeT87bh95OGNyZPhcTNXj1NW7RuBCsyN/o0jlpcV8Qyq46cDfL" crossorigin="anonymous"></script>

<!-- AOS 3.0.0-beta.6 plugin JS (Animations) -->
<script src="https://unpkg.com/aos@next/dist/aos.js"></script>
<script>
    AOS.init();
</script>

    '''
    return content
    





def json_editor():

    content = '''

        <script src="https://cdnjs.cloudflare.com/ajax/libs/ace/1.13.0/ace.js" integrity="sha512-btmS7t+mAyZXugYonCqUwCfOTw+8qUg9eO9AbFl5AT2zC1Q4we+KnCQAq2ZITQz1c9/axyUNYaNhGWqxfSpj7g==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
        
        <div id="editor-container" class="app_editor_wrapper">
            <div id="editor" class="app_editor">
{
    "@type": "schema:WebPage",
    "schema:url": "https://"
}
            </div>
        </div>
        <div>
        <button type="button" class="btn btn-primary" onclick="buttonClick()">Post</button>
        </div>

        
        
        <script>            
            var editor = ace.edit("editor");
            editor.setTheme("ace/theme/dawn");
            editor.session.setMode("ace/mode/json");

            editor.showCommandLine = function(val) {
                this.cmdLine.focus();
                if (typeof val == "string")
                    this.cmdLine.setValue(val, 1);
            };

        </script>


        <script>
        function buttonClick() {
            console.log('Button clicked')
            
            //var json_object = JSON.stringify(editor.getValue())
            var json_object = editor.getValue()
            console.log('json:');
            console.log(json_object);
            return new Promise(resolve => {
        
                fetch('https://engine.vlln.co/api', {
                    method: 'POST', // or 'PUT'
                    headers: {
                        'Content-Type': 'application/json',
                    },
                    body: json_object,
                })
                    .then((response) => response.json())
                    .then((data) => {
                        console.log('Success post record');
                    })
                    .catch((error) => {
                    });
                console.log('post');
                //console.log(data);
                resolve('Done');
            });

        }

        </script>
        

    '''
    return content




import json

from kraken_html.library import html
from kraken_html.library import parts
from kraken_html.library import html_record



def testimonial(record, mode='light'):
    """

    input record: https://schema.org/Rating
    """


    # Handle lists:
    if isinstance(record, list):
        cards = []
        for i in record:
            cards.append(testimonial(i))
        return html.cardgrid(cards)

    # Initialize body content
    body_content = ''
    
    # Get star rating
    body_content += _get_rating(record, mode)

    # Get author
    body_content += _get_citation(record, mode)

    # Get citation content
    body_content += _get_name_title(record, mode)


    # Put into card
    content = html.card(None, body_content, None)
    

    return content


def _get_rating(record, mode='light'):
    """
    """
    rating_value = record.get('ratingValue', None) 

    if not rating_value:
        return ''

    content = f'<p>{html.rating(rating_value)}</p>'

    return content


def _get_citation(record, mode='light'):
    """
    """

    ratingExplanation = record.get('ratingExplanation', None) 

    content = f'<p>"{ratingExplanation}"</p>'

    return content
        


def _get_name_title(record, mode='light'):
    """
    """
    author = record.get('author', {}) 
    given_name = author.get('givenName', None)
    family_name = author.get('familyName', None)
    author_name = f'{given_name} {family_name}'
    jobTitle = author.get('jobTitle', None)
    

    content = ''
    if author_name is not None and author_name != '':
        content += f'<h4>{author_name}</h4>'

    if jobTitle is not None and jobTitle != '':
        content += f'<p class="fs-6 mb-0">{jobTitle}</p>'

    return content





def test_testimonial():
    """
    """

    content = html.section(
        [
            _test_code_content(),
            _test_testimonial_single(),
            _test_testimonial_multiple()
        ]
    )
    
    

    return content

def _test_code_content():
    """
    """
    code_content = '''
    from kraken_html.library import widgets
    record = {...}                          # Single or list of schema.org/rating
    content = widgets.testimonial(record)   # html content
    return Response(content)                
    '''
    content = html.section(
        [
            html.title('Code', 'h2'),
            html.code(code_content)
        ]
    )
    return content

def _test_testimonial_single():
    
    
    record = _get_record()

    content = html.section(
        [
            html.div(
                html.title('Single record', 'h2')
            ),
            html.div(
                html.accordion('Record', html_record.to_json(record))
            ),
            html.div(
                html.title('widget', 'h3')
            ),
            html.div(
                testimonial(record)
            )
        ]
    )
   
    return content


def _test_testimonial_multiple():

    record = _get_record()
    record = [record, record, record]

    content = html.section(
        [
            html.div(
                html.title('Multiple records', 'h2')
            ),
            html.div(
                html.accordion('Records', html_record.to_json(record))
            ),
            html.div(
                html.title('widget', 'h3')
            ),
            html.div(
                testimonial(record)
            )
        ]
    )

    return content



def _get_record():
    record = {
        '@type': 'Rating',
        '@id': 'test_testimonial',
        'author': {
            '@type': 'person',
            '@id': 'reviewer1',
            'givenName': 'John',
            'familyName': 'Smith',
            'jobTitle': 'Director engineering'
        },
        'ratingValue': 3.5,
        'ratingExplanation': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. '
    }

    return record


import uuid

def accordion(title, content, expanded = False):
    """
    """

    if isinstance(content, list):
        content = ' '.join(content)

    
    component_id1 = 's' + str(uuid.uuid4())
    component_id2 = 's' + str(uuid.uuid4())
    component_id3 = 's' + str(uuid.uuid4())

    #component_id1 = 'aaa'
    #component_id2 = 'bbb'

    if expanded == True:
        expanded = 'aria-expanded="true"'
        collapsed = ''
        show = 'show'
    else:
        expanded = 'aria-expanded="false"'
        collapsed = 'collapsed'
        show = ''


    content = '''

            <div class="accordion" id="{component_id1}">
                <div class="accordion-item">
                    <h2 class="accordion-header" id="{component_id3}">
                        <button 
                            class="accordion-button {collapsed}" 
                            type="button" 
                            data-bs-toggle="collapse" 
                            data-bs-target="#{component_id2}" 
                            aria-controls="{component_id2}"
                            {expanded}
                            >
                            
                                {title}
                        </button>
                    </h2>
                    <div 
                        id="{component_id2}" 
                        class="accordion-collapse collapse {show}" 
                        aria-labelledby="{component_id2}" 
                        >
                        
                        <div class="accordion-body">
                            
                            {content}
                        </div>
                    </div>
                </div>
            
            </div>

    
        '''.format(
            title = title,
            content = content,
            component_id1 = component_id1,
            component_id2 = component_id2,
            component_id3 = component_id3,
            expanded = expanded,
            collapsed = collapsed,
            show = show
            )

    return content








def image(url, title = None, size = None, card = False, action_id=None):


    if not url:
        return ''

    if not isinstance(url, str):
        return ''
    
    if url.startswith('<'):
        return ''
    
    if isinstance(url, list):
        url = url[0]

    class_items = ''
    if card == True:
        class_items = 'card-img-top'
    
    size_html = ''
    if size == 'xxs':
        size_html = 'style="max-height:50px"'
    if size == 'xs':
        size_html = 'style="max-height:100px"'
    if size == 'sm':
        size_html = 'style="max-height:300px"'
    if size == 'md':
        size_html = 'style="max-height:600px"'
    if size == 'lg':
        size_html = 'style="max-height:1200px"'


    action_html = None
    if action_id:
        action_html = 'data-bs-toggle="modal" data-bs-target="#{action_id}"'.format(action_id=action_id)

    
    content = '<img loading="lazy" class="img-fluid {class_items}" src="{url}" {size_html} {action_html}>'.format(url=url, title=title, size_html=size_html, action_html=action_html, class_items=class_items)

    return content





from kraken_html.library.sections.pricing import pricing
from kraken_html.library.sections.hero import hero
from kraken_html.library.sections.nav_bar import nav_bar
from kraken_html.library.sections.search import search


import datetime

def footer(legal_name, brand, item_links):
    '''
    '''


    # Get current year
    today = datetime.date.today()
    year = today.year

    # Transform links
    links = get_footer_links(item_links)

    # Combine content
    
    
    content = f'''

        <div class="container-fluid">
          <footer class="d-flex flex-wrap justify-content-between align-items-center py-3 my-4 border-top">
            
            <p class="col-md-4 mb-0 text-body-secondary">&copy; 2023 {legal_name}</p>
        
            <a href="/" class="col-md-4 d-flex align-items-center justify-content-center mb-3 mb-md-0 me-md-auto link-body-emphasis text-decoration-none">
              
            </a>
        
            <ul class="nav col-md-4 justify-content-end">
              {links}
            </ul>
          </footer>
        </div>

    '''
    return content


def get_footer_links(item_links):
    '''
    '''
    content = ''
    for link in item_links:
        content += footer_link(link.get('text'), link.get('url'))
    return content
        
    

def footer_link(text, url):
    '''
    '''

    content = f'''
        <li class="nav-item">
            <a href="{url}" class="nav-link px-2 text-body-secondary">
                {text}
            </a>
        </li>
    '''
    return content




def card(header=None, body=None, footer=None):
    """content - html fo the content
    title - text
    url - url for link
    """

    content = f'''
<div class="card h-100">
    {header_content(header)}
    {body_content(body)}
    {footer_content(footer)}
</div>    
'''
    return content



def header_content(initial_content):
    """
    """
    if initial_content is None or initial_content == '':
        return ''
    
    content = f'''
<div class="card-header">
    {initial_content}
</div>
    '''
    return content


def body_content(initial_content):
    """
    """
    if initial_content is None or initial_content == '':
        return ''

    content = f'''
<div class="card-body">
    {initial_content}
</div>
    '''
    return content


def footer_content(initial_content):
    """
    """
    if initial_content is None or initial_content == '':
        return ''

    content = f'''
<div class="card-footer">
    {initial_content}
</div>
    '''
    return content







def get_image(url, title = None, size = None, action_id=None):

    if isinstance(url, list):
        url = url[0]


    size_html = ''
    if size == 'xxs':
        size_html = 'style="max-height:50px"'
    if size == 'xs':
        size_html = 'style="max-height:100px"'
    if size == 'sm':
        size_html = 'style="max-height:300px"'
    if size == 'md':
        size_html = 'style="max-height:600px"'
    if size == 'lg':
        size_html = 'style="max-height:1200px"'


    action_html = None
    if action_id:
        action_html = 'data-bs-toggle="modal" data-bs-target="#{action_id}"'.format(action_id=action_id)

    
    content = '<img loading="lazy" class="img-fluid" src="{url}" {size_html} {action_html}>'.format(url=url, title=title, size_html=size_html, action_html=action_html)

    return content


def get_video(url, text = None):

    if isinstance(url, list):
        url = url[0]
    
    if not url:
        return ''
    if len(url) < 5:
        return ''

    html_thumb = '''
            <video class="video-fluid z-depth-1" playsinline controls>
            <source src="{url}#t=2" type="video/mp4">
            <source src="{url}#t=2" type="video/ogg">
            </video>
            
        '''.format(url=url, text=text)

    html_thumb = '''
            <video class="video-fluid z-depth-1" playsinline controls>
            <source src="{url}#t=2">
            </video>
            
        '''.format(url=url, text=text)
    zzhtml_thumb = '''
            <div class="embed-responsive embed-responsive-21by9">
        <iframe class="embed-responsive-item" loading="lazy" src="{url}" allowfullscreen></iframe>
        </div>
            
        '''.format(url=url, text=text)




    #html = '<a href="{url}">{html_thumb}</a>'.format(url=url, html_thumb=html_thumb)


    return html_thumb




from .html_nav import html_nav
from .footer import footer
from .page import page


def website(config, page_title, main_content):
    """Given a config file, returns a webpage
    """
    

    # Get menu
    organization = config.get('subjectOf', {})
    brand = organization.get('brand', {})
    brand_name = brand.get('name', None)
    menu_items = config.get('menu', [])
    menu_content = html_nav(brand_name, menu_items)


    # Get footer
    organization = config.get('subjectOf', None)
    legal_name = organization.get('legalName', None)
    brand_name = brand.get('name', None)
    footer_items = config.get('footer', [])
    footer_content = footer(legal_name, brand_name, footer_items)

    # aggregate content
    content = menu_content + main_content 
    
    # Get page 
    page_content = page(page_title, menu_content, main_content, footer_content)

    return page_content

def get_config():
    """Returns a sample config file
    """
    
    record= {
        '@type': 'website',
        'name': 'Name of the website',
        'subjectOf': {
            '@type': 'organization',
            'name': 'Test_org_name',
            'legalName': 'Test_legal_name',
            'brand': {
                '@type': 'brand',
                'name': 'brand_name',
                'logo': 'http://logo_url.com'
            }
        },
        'menu': [
            {'text': 'Home', 'url': '/'},
            {'text': 'Products', 'url': '/products'},
            {'text': 'About us', 'url': '/about_us'}
        ],
        'footer': [
            {'text': 'Home', 'url': '/'},
            {'text': 'Products', 'url': '/products'},
            {'text': 'About us', 'url': '/about_us'}
        ]
    }




from kraken_html.library.widgets.testimonial import *
from kraken_html.library.widgets.record import *
from kraken_html.library.widgets.records import *
from kraken_html.library.widgets.person import *
from kraken_html.library.widgets.map import *





def get_list(record, level=0):

    sub_content = ''
    print('----')
    print(record)
    print('----')
    for k, v in record.items():
        sub_content += get_list_row(k, v)


    content = f'''
        
        <div class="container-fluid">
        <dl class="row mb-0">
            {sub_content}
        </dl>
        </div>
        
    
    '''
    return content


def get_list_row(key, value, leve=0):
    """
    """
    content = f'''

        <dl class="row mb-0">
          <dt class="col-sm-2 mb-0">{key}</dt>
          <dd class="col-sm-10 mb-0">
          {get_list_items(value)}
          </dd>
        </dl>
    
    
    '''
    return content

def get_list_items(value, level=0):
    """
    """
    value = value if isinstance(value, list) else [value]
    content = '<ul class="list-unstyled mb-0">'

    for v in value:
        if 1==0:
            if isinstance(v, dict) and level < 2:
    
                content += f'''
                <details>
                    <summary>{str(v.get('@type', None)) + '/' + str(v.get('@id', None))}</summary>
    
                    <p>{get_list(v, level + 1)}</p>
                </details>
                
                
                '''
                
                #content += get_list(v, level + 1)
    
            else:
                if len(value) == 1:
                    content += v
                else:
                    content += f'''
                        <li mb-0>{v}</li>
                '''
        if not v:
            continue
            
        if len(value) == 1:
            content += v
        else:
            content += f'''
                <li mb-0>{v}</li>
                '''
            
    content += '</ul>'
    return content










def get_table_row(header, record, thumbnail= False):
     #Return html table line from record and header
        
    # Build row
    row = '<tr>'
    for h in header:
        # Build cell

        # Get data
        value = record.get(h, '')


        if isinstance(value, list):
            
            if thumbnail:
                if len(value) > 0:
                    value = value[0]
                else:
                    value = ''
            else:
                value = str(len(value)) + ' entities'

        content = value

        cell = '<td>' + str(content) + '</td>'
        row += cell

    row += '</tr>'

    return row




def table(records, header = None, header_title = None,thumbnail = True):
    """Converts json records to html table
    """

    if records is None:
        return ''
    
    if not header:
        header = []
        for record in records:
            if record is None:
                continue
            for k in record.keys():
                if k not in header:
                    header.append(k)
        header.sort()
    
    if not header_title:
        header_title = header


    if 1==0:
        if 'schema:contentUrl' in header:
            header.remove('schema:contentUrl')
            header.insert(0,'schema:contentUrl')
    
        if 'schema:contenturl' in header:
            header.remove('schema:contenturl')
            header.insert(0,'schema:contenturl')


    # Build rows
    rows = ''
    for record in records:
        rows += get_table_row(header, record, thumbnail)
        
    table_rows = '<tbody>' + rows + '</tbody>'

    # Build header

    header_text = '<thead class="table-dark"><tr>'
    for h in header_title:
        cell = '<th>' + h + '</th>'
        header_text += cell
    header_text += '</tr></thead>'


    # Build table
    table = f'<div class="table-responsive"><table class="table table-sm table-responsive"> {header_text} {table_rows} </table></div>'


    return table



import json

def schema_record(record):
    """
    """
    json_content = json.dumps(record, default=str, indent=4)
    schema_content = f'<script type="application/ld+json">{json_content}</script>'

    return schema_content



from kraken_html.library import html

def search(action, theme='light'):
    """Returns a html section for performing search
    """

    # Build search form

    record_id = action.get('@id', None)
    url = action.get('url', None)
    name = action.get('name', None)
    
    
    content = f'''
    
<section>
    <div class="px-4 py-5 my-5 text-center bg-{theme} text-bg-{theme}" data-bs-theme="{theme}" data-aos="fade-down" data-aos-delay="200">

    <form action="{url}" method="post">
    <div class="row">
        <div class="col">
            <input type="text" class="form-control" id="{record_id}" name="{record_id}" placeholder="" aria-label="query">
        </div>
            <div class="col">
                <button type="submit" class="btn btn-primary">Submit</button>
            </div>
        </div>
    </form>
    </div>
</section>



</div>
</section>
    
    
    '''

    return content





def map(record):

    API_KEY = 'AIzaSyCdVL64h1rNYu83c8pHBiNy5E1UfiCpc8g'

    street = record.get('streetAddress', '')
    city = record.get('addressLocality', '')
    state = record.get('addressRegion', '')
    country = record.get('addressCountry', '')
    postalCode = record.get('postalCode', '')


    query = f'''{street}, {city}, {state},{country},{postalCode}'''
    
    content = f'''
<iframe
  width="600"
  height="450"
  style="border:0"
  loading="lazy"
  allowfullscreen
  referrerpolicy="no-referrer-when-downgrade"
  src="https://www.google.com/maps/embed/v1/place?key={API_KEY}
    &q={query}">
</iframe>
    
    '''
    content = f'''
    <iframe
      width="600"
      height="450"
      style="border:0"
      loading="lazy"
      allowfullscreen
      referrerpolicy="no-referrer-when-downgrade"
      src="https://www.google.com/maps/embed/v1/place?key={API_KEY}
        &q={query}">
    </iframe>

        '''
    return content

    content = f'''
<script src="https://polyfill.io/v3/polyfill.min.js?features=default"></script>
<script>
    let map;
    
    async function initMap() {{
      // The location of Uluru
      const position = {{ lat: -25.344, lng: 131.031 }};
      // Request needed libraries.
      //@ts-ignore
      const {{ Map }} = await google.maps.importLibrary("maps");
      const {{ AdvancedMarkerElement }} = await google.maps.importLibrary("marker");
    
      // The map, centered at Uluru
      map = new Map(document.getElementById("map"), {{
        zoom: 4,
        center: position,
        mapId: "DEMO_MAP_ID",
      }});
    
      // The marker, positioned at Uluru
      const marker = new AdvancedMarkerElement({{
        map: map,
        position: position,
        title: "Uluru",
      }});
    }}
    
    initMap();

</script>
    <div id="map"></div>

    <!-- prettier-ignore -->
    <script>
    (g=>{{var h,a,k,p="The Google Maps JavaScript API",c="google",l="importLibrary",q="__ib__",m=document,b=window;b=b[c]||(b[c]={{}});var d=b.maps||(b.maps={{}}),r=new Set,e=new URLSearchParams,u=()=>h||(h=new Promise(async(f,n)=>{{await (a=m.createElement("script"));e.set("libraries",[...r]+"");for(k in g)e.set(k.replace(/[A-Z]/g,t=>"_"+t[0].toLowerCase()),g[k]);e.set("callback",c+".maps."+q);a.src=`https://maps.${{c}}apis.com/maps/api/js?`+e;d[q]=f;a.onerror=()=>h=n(Error(p+" could not load."));a.nonce=m.querySelector("script[nonce]")?.nonce||"";m.head.append(a)}}));d[l]?console.warn(p+" only loads once. Ignoring:",g):d[l]=(f,...n)=>r.add(f)&&u().then(()=>d[l](f,...n))}})
        ({{key: "AIzaSyB41DRUbKWJHPxaFjMAwdrzWzbVKartNGg", v: "weekly"}});
        </script>
    
    '''
    return content
    
    content = f'''
    <script>
        const CONFIGURATION = {{
            "locations": [
              {{
                "title":"{street}",
                "address1":"{street}",
                "address2":"{city}, {state} {postalCode}, {country}"
                }}
            ],
            "mapOptions": {{
                "center":{{"lat":38.0,"lng":-100.0}},
                "fullscreenControl":true,
                "mapTypeControl":false,
                "streetViewControl":false,
                "zoom":4,
                "zoomControl":true,
                "maxZoom":17,
                "mapId":""
                }},
        "mapsApiKey": "AIzaSyCdVL64h1rNYu83c8pHBiNy5E1UfiCpc8g",
        "capabilities": {{
            "input":false,
            "autocomplete":false,
            "directions":false,
            "distanceMatrix":false,
            "details":false,
            "actions":false
          }}
        }};
    </script>
    <script type="module">
        document.addEventListener('DOMContentLoaded', async () => {{
            await customElements.whenDefined('gmpx-store-locator');
            const locator = document.querySelector('gmpx-store-locator');
            locator.configureFromQuickBuilder(CONFIGURATION);
        }});
    </script>

    <!-- Please note unpkg.com is unaffiliated with Google Maps Platform. -->
    <script type="module" src="https://unpkg.com/@googlemaps/extended-component-library@0.6"></script>

    <!-- Uses components from the Extended Component Library; see
         https://github.com/googlemaps/extended-component-library for more information
         on these HTML tags and how to configure them. -->
         
    <gmpx-api-loader key="AIzaSyCdVL64h1rNYu83c8pHBiNy5E1UfiCpc8g" solution-channel="GMP_QB_locatorplus_v10_c"></gmpx-api-loader>
    
    <gmpx-store-locator map-id="DEMO_MAP_ID"></gmpx-store-locator>

    '''
    
    return content





def get_modal(modal_id, content):
    

    content = '''
        <!-- Modal -->
                <div class="modal fade" id="{modal_id}" tabindex="-1" aria-labelledby="exampleModalLabel" aria-hidden="true" role="dialog">
                  <div class="modal-dialog modal-lg">
                    <div class="modal-content">
                      {content}
                     
                    </div>
                  </div>
                </div>

    '''.format(
        modal_id = modal_id, 
        content=content
    )
    return content



from kraken_html.library import sections
from kraken_html.library import parts
from kraken_html.library import config

def base(website_record, page_content, schema_record={}, language='en'):
    """
    """
    
    name = website_record.get('name', None)
    favicon_url = website_record.get('thumbnailUrl', None)
    

    
    website_record = {
        '@type': 'WebSite',
        'hasPart':[

            {
                '@type': 'WPHeader',
                'hasPart': [
                    {
                        '@type': 'SiteNavigationElement',
                        'headline': 'Home',
                        'url': '/main'
                    }
                    
                ]
            },
            {
                '@type': 'WPFooter',
                'hasPart': [
                    {
                        '@type': 'SiteNavigationElement',
                        'headline': '',
                        'url': '/main'
                    }
                ]
            }
        ]
        
    }

    content = f'''
    
<!DOCTYPE html>
<html  lang="{language}">
    
    <head>
        <meta charset="utf-8" >
        <title>{name}</title>
        <link rel="icon" href="{favicon_url}">
        
        <!-- bootstrap css reqs -->
        {config.bootstrap_header()}

        <!-- schema.org info -->
        {config.schema_record(schema_record)}
        
    </head> 
    
    <body>

        <!-- nav_bar -->
        {sections.nav_bar(website_record)}

        <!-- actual content of the web page -->
        {page_content}

        <!-- js scripts for bootstrap -->
        {config.bootstrap_body()}

    </body>

</html>

    
    '''
    return content



import json
import copy

def to_list(record):
    """Convert dict to html
    """
    record = copy.deepcopy(record)

    if isinstance(record, list):
        count = 0
        content = '<dl class="row">'
        for i in record:
            html_v = to_list(i)
            content += f'''
            <dt class="col-sm-3">{str(count)}</dt>
            <dd class="col-sm-9">{html_v}</dd>
            '''
            count += 1
        content += '</dl>'
        return content
    
    elif isinstance(record, dict):
        content = '<dl class="row">'
        for k, v in record.items():
            html_v = to_list(v)
            content += f'''
            <dt class="col-sm-3">{k}</dt>
            '''
            v = v if isinstance(v, list) else [v]
            for v1 in v:
                html_v = to_list(v1)
                content += f'''
                <dd class="col-sm-9">{html_v}</dd>
                '''
        content += '</dl>'
        return content
    
    else:
        return record
        
def to_json(record):
    """Convert dict to html
    """

    record = copy.deepcopy(record)
    
    content = json.dumps(record, default=str, indent=4)
    content = content.replace('\n', '<br>')
    content = content.replace('    ', '&nbsp;&nbsp;&nbsp;&nbsp;')
    return content




from kraken_html.library import html
from kraken_html.library.html_record import record_names as thing_names
import uuid

def html_record(value, expand=False):
    # Returns dict with enhanced values


    if not value:
        return None

    if isinstance(value, list):
        new_v = [html_record(x, expand) for x in value]
        return new_v


    # Buidld new dict
    new_record = {}
    for key, values in value.items():

        new_record[key] = []
        values = [values] if not isinstance(values, list) else values



        for v in values:

            new_value = v

            if isinstance(v, dict):
                if '@type' in v.keys() and key:
                    if not expand:
                        new_value = html_value_record_ref(v)
                    else:
                        new_value = html_record_expandable_section(v)

            elif key and 'thumbnailUrl' in key:
                new_value = html_value_thumbnail(v)

            elif key and 'url' in key:
                new_value = html_value_url(v)

            elif key == '@type':
                new_value = html_value_type(v)

            elif key == '@id':
                new_value = html_value_id(value)

            else:
                new_v = v
                new_value = new_v

            new_record[key].append(new_value)

    return new_record



def html_record_expandable_section(value):
    """
    """
    summary = html_value_record_ref(value)
    new_record = html_record(value, True)

    details = html.get_list(new_record)

    content = html.summary(summary, details)
    return content



def html_value_record_ref(value):

    name = thing_names.get(value)

    t = value.get('@type', None)
    i = value.get('@id', None)



    t = t if not isinstance(t, list) else t[0]
    i = i if not isinstance(i, list) else i[0]

    link = '/' + str(t) + '/' + str(i)

    if not name:
        name = link

    for k, v in value.items():
        if 'name' in k.lower():
            if not isinstance(v, list):
                v = [v]
            if len(v) > 0:
                name = v[0]

    new_v = html.link(link, name)
    return new_v


def html_value_type(record_type):
    # Add link to types
    if isinstance(record_type, list) and len(record_type) > 0:
        record_type = record_type[0]
    new_v = html.link('/' + str(record_type), record_type)
    return new_v

def html_value_id(record):
    # Add link to types
    record_type = record.get('@type', None)
    if isinstance(record_type, list) and len(record_type) > 0:
        record_type = record_type[0]
    record_id = record.get('@id', None)
    if isinstance(record_id, list) and len(record_id) > 0:
        record_id = record_id[0]

    # Escape record id


    new_v = html.link('/' + str(record_type) + '/' + str(record_id), str(record_id))
    return new_v

def html_value_url(value):
    # Add link to url
    new_v = html.link(value, value)
    return new_v

def html_value_thumbnail(url):
    # Returns image with a link

    modal_id = 'ref_' + str(uuid.uuid4())
    image = html.image(url, None, 'xxs', None, modal_id)

    modal_content = html.image(url, None, 'xxl')
    modal = html.get_modal(modal_id, modal_content)

    content = image + modal



    return content


def div(html, alignment=None, minimum_height=0, padding=0):

    if isinstance(html, list):
        print('is list')
        html = ' '.join(html)

    
    minimum_height = str(minimum_height)
    
    if alignment == 'right':
        content = f'''
        <div class="container-fluid d-flex justify-content-end min-vh-{minimum_height} {'p-' + str(padding) if padding else ''}">
            {html}
        </div>
        '''

    else:
        content = f'''
            <div class="container-fluid min-vh-{minimum_height} {'p-' + str(padding) if padding else ''}">
                {html}
            </div>
            '''

    return content







def bootstrap_header():

    
    
    content = f'''

<!-- AOS 3.0.0-beta.6 plugin CSS (Animations) -->
<link rel="stylesheet" href="https://unpkg.com/aos@next/dist/aos.css" />

<!-- Google Fonts -->
<link href="https://fonts.googleapis.com/css2?family=DM+Sans:wght@100;200;300;400;500;600;700;800;900&family=DM+Sans:wght@100;200;300;400;500;600;700;800;900&subset=latin&display=swap" rel="stylesheet" id="sf-google-fonts">
    
<!-- Startup CSS + Bootstrap CSS (Styles for all blocks)  -->
<link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.2/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-T3c6CoIi6uLrA9TneNEoa7RxnatzjcDSCmG1MXxSR1GAsXEV/Dwwykc2MPK8M2HN" crossorigin="anonymous">

<!-- Bootstrap icons  -->
<link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bootstrap-icons@1.11.2/font/bootstrap-icons.min.css">
    
<!-- jQuery 3.6.0 -->
<script src="https://cdnjs.cloudflare.com/ajax/libs/jquery/3.6.0/jquery.min.js"></script> 

            
    '''
    return content




def rating(ratingValue):
    """Returns a star rating
    """

    if not isinstance(ratingValue, (int, float)):
        return ''
    
    if ratingValue > 5 and ratingValue <= 10:
        ratingValue = ratingValue / 2

    if ratingValue > 10:
        ratingValue = ratingValue / 10
    
    content = ''
    count = 0
    
    while count < int(ratingValue):
        content += '<i class="bi bi-star-fill text-warning"  fill="currentColor" ></i>'
        count += 1


    if int(ratingValue) != ratingValue:
        content += '<i class="bi bi-star-half text-warning" fill="currentColor" ></i>'
        count += 1
    
    
    while count < 5:
        content += '<i class="bi bi-star text-warning" fill="currentColor" ></i>'
        count += 1
       
    return content





def section(content=None):
    """
    """
    
    if isinstance(content, list):
        content = ' '.join(content)

    
    section_content = f'''
<section>
    {content}
</section>
    '''

    return section_content


from kraken_html.library import html


def map(record, mode='light', language='EN-US'):
    """
    """



    # 
    street = record.get('streetAddress', '')
    city = record.get('addressLocality', '')
    state = record.get('addressRegion', '')
    country = record.get('addressCountry', '')
    postalCode = record.get('postalCode', '')


    
   
    # Get map
    map_content = html.map(record) 
    
    

    # Generate list
    address_line1 = f'{city}, {state}, {postalCode}'
    address_content = ''
    address_content += _get_list_element(address_line1)
    address_content += _get_list_element(country)
    

    
    # Generate content
    content = f'''
    
<div class="card mb-3" style="height: 500px;" >
  <div class="row g-0 align-items-center">
    <div class="col-md-8">
    <div class="container-fluid google-maps text-center">
      {map_content}
      </div>
    </div>
    <div class="col-md-4">
      <div class="card-body">
        <h5 class="card-title">{str(street)}</h5>
        <ul class="list-unstyled">
            {address_content}
        </ul>
        </small></p>
      </div>
    </div>
  </div>
</div>
    '''
    return content



def _get_list_element(content):
    """
    """

    if not content or content == '':
        return ''

    content = str(content)
    
    content = f'''
    <li>{content}</li>
    '''
    return content






def iframe(url, width=510, height=400):
    """
    """

    if width and height:
        size_content = 'width="{width}" height="{height}"'.format(width=width, height=height)
    
    content = '''
        <iframe src="{url}" frameborder="0" {size_content} allowfullscreen="allowfullscreen"></iframe>
    '''.format(url=url, size_content = size_content)

    return content

    '''
    <iframe src="http" frameborder="0" width="510" height="400" scrolling="no" allowfullscreen="allowfullscreen"></iframe>
    
    '''



from kraken_html.library import html
from kraken_html.library import parts
from kraken_html.library import html_record



def record_item(record, mode='light'):
    """

    input record: https://schema.org/Rating
    """

    content = html.section(
        [
            html.title(
                html_record.record_names.get(record),
                'h1'
            ),
            html_record.to_list(
                html_record.html_record(record, True)
            )
        ]
    )
    
    return content
    
    
def test_record():
    """
    """
    
    record = _get_test_record()

    content = html.section(
        [
            html.title(
                'Record widget', 'h1'
            ),
            
            html.div(
                [
                    html.title(
                        'Code', 'h2'
                    ),
                    html.code(
                        _get_code_content()
                    )
                ]
            ),
            html.div(
                [
                    html.title(
                        'Record Record', 'h2'
                    ),
                    html.accordion('Record example', html_record.to_json(record))
                ]
            ),
            html.div(
                [
                    html.title(
                        'Record widget', 'h2'
                    ),
                    record_item(record)
                ]
            )
        ]
    )

    return content
        



def _get_test_record():
    record = {
        '@type': 'Rating',
        '@id': 'test_testimonial',
        'author': {
            '@type': 'person',
            '@id': 'reviewer1',
            'givenName': 'John',
            'familyName': 'Smith',
            'jobTitle': 'Director engineering'
        },
        'ratingValue': 3.5,
        'ratingExplanation': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. '
    }

    return record

def _get_code_content():
    code_content = '''
        from kraken_html.library import widgets
        record = {...}                          # Single or list of schema.org/rating
        content = widgets.testimonial(record)   # html content
        return Response(content)                
    '''
    return code_content





def hero(record, theme='dark'):
    '''
    '''

    content = f'''

<section>
<div class="px-4 py-5 my-5 text-center bg-{theme} text-bg-{theme}" data-bs-theme="{theme}" data-aos="fade-down" data-aos-delay="200">
    <img class="d-block mx-auto mb-4" src="/docs/5.3/assets/brand/bootstrap-logo.svg" alt="" width="72" height="57">
    <h1 class="display-5 fw-bold text-body-emphasis">{record.get('headline', None)}</h1>
    <div class="col-lg-6 mx-auto">
      <p class="lead mb-4" >{record.get('text', None)}</p>
      <div class="d-grid gap-2 d-sm-flex justify-content-sm-center">
        <button type="button" class="btn btn-primary btn-lg px-4 gap-3">Primary button</button>
        <button type="button" class="btn btn-outline-secondary btn-lg px-4">Secondary</button>
      </div>
    </div>
  </div>
  </section>
    
    '''
    return content


"""
Obtain the name of the thing based on convention if name is missing

"""





def get(record):
    """Obtain the name of the thing based on convention if name is missing

    """

    def get_from_list(key, record):

        if not record:
            return None

        if not isinstance(record, dict):
            return None

        value = record.get(key, None)
        if isinstance(value, list):
            if len(value) > 0:
                value = value[0]
            else:
                value = None
        return value

    record_type = get_from_list('@type', record)
    record_id = get_from_list('@id', record)
    name = get_from_list('name', record)
    url = get_from_list('url', record)



    # Return name if exists in record
    if name:
        return name

    if not record_type:
        return None

    # Get name based on type
    if record_type == 'person':
        name = _get_name_from_person(record)
    if record_type == 'address':
        name = _get_name_from_address(record)

    # If empty, use url
    if not name:
        url = record.get('url', None)
        name = url

    # If empty, use type / id
    if not name:
        name = f'{record_type}/{record_id}'

    return name



def _get_name_from_person(record):
    """
    """
    def get_from_list(key, record):

        if not record:
            return None

        if not isinstance(record, dict):
            return None

        value = record.get(key, None)
        if isinstance(value, list):
            if len(value) > 0:
                value = value[0]
            else:
                value = None
        return value

    fname = get_from_list('givenName', record)
    lname = get_from_list('familyName', record)


    name = ' '.join([fname, lname])

    return name



def _get_name_from_address(record):
    """
    """
    def get_from_list(key, record):

        if not record:
            return None

        if not isinstance(record, dict):
            return None

        value = record.get(key, None)
        if isinstance(value, list):
            if len(value) > 0:
                value = value[0]
            else:
                value = None
        return value

    values = []
    for k in ['streetAddress', 'addressLocality', 'addressRegion', 'addressCountry', 'postalCode']:
        v = get_from_list(k, record)
        if v:
            values.append(v)
    name = ','.join(values)

    return name



    
import copy
from kraken_html.helpers import json
import os
import pkg_resources
from kraken_html import kraken_html as m

"""
Notes:
To access files in data directory, use:
new_path = pkg_resources.resource_filename('kraken_html', old_path)

"""
    
class Html:
    """
    The Vehicle object contains a lot of vehicles

    Args:
        arg1 (str): The arg is used for...
        arg2 (str): The arg is used for...
        arg3 (str): The arg is used for...

    Attributes:
        record (dict): This is where we store attributes
        json (str): Record in json format
        
    """

    def __init__(self):
        self._record = {}
        

    def __str__(self):
        """
        """
        return str(self._record)

    
    def __repr__(self):
        """
        """
        return str(self._record)

    
    def __eq__(self, other):
        """
        """
        if type(self) != type(other):
            return False
            
        if self._record == other._record:
            return True
        return False

    def __gt__(self, other):
        """
        """
        return True

        
    def set(self, property, value):
        """
        """
        self._record[property] = value
        return True

    
    def get(self, property):
        """
        """
        return self._record.get(property, None)

    
    def load(self, value):
        """
        """
        self._record = value
        return True


    def dump(self): 
        """
        """
        return copy.deepcopy(self._record)
        

    def set_json(self, value):
        """
        """
        record = json.loads(value)
        self.load(record)
        return True

    def get_json(self):
        """
        """
        return json.dumps(self.dump())

    @property
    def record(self):
        return self.dump()

    @record.setter
    def record(self, value):
        return self.load(value)
    
    @property
    def json(self):
        return self.get_json()

    @json.setter
    def json(self, value):
        return self.set_json(value)
        

    def search(self, value=None, value2=None):
        return 1





def breadcrumb(path):
    """
    """

    if not isinstance(path, list):
        path = [path]

    new_path = path

    items = []

    # Add home
    item_home = '<li class="breadcrumb-item"><a href="/">Home</a></li>'
    items.append(item_home)
    
    # Add items
    url = ''
    while new_path:
        item = new_path.pop(0)
        url = url + '/' + item
        if len(new_path) >= 1:
            item = '<li class="breadcrumb-item"><a href="{url}">{item}</a></li>'.format(item=item, url=url)
        else:
            item = '<li class="breadcrumb-item active" aria-current="page">{item}</li>'.format(item=item)
            
        items.append(item)

    # Merge items
    item_content = ' '.join(items)

    # Generate content
    content = '''
        <nav aria-label="breadcrumb">
          <ol class="breadcrumb">
            {item_content}
          </ol>
        </nav>
    
    '''.format(item_content=item_content)
    
    return content